﻿/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|             File Name: ModbusBoards.h                                                                                 |
|                Author: Michael Krämer                                                                                 |
|                 Email: Michael@dk8pp.de                                                                               |
|                  Date: 02.04.2022                                                                                     |
|                  Tool: AVR Studio 7.0 and Visual Studio                                                               |
|                Device: ATTiny, ATMEGA, ATXMEGA                                                                        |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Include file for all modbus board projects to define board version and purpose                              |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef MODBUSBOARDS

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define MODBUSBOARDS 1

/* Module type */
#define MT_UNKNOWN    0x0000   /* ATTINY1634 Board V1.0, Antenna switch V 1.0 */
#define MT_ANTSW_V10  0x0101   /* ATTINY1634 Board V1.0, Antenna switch V 1.0 */
#define MT_ANTSW_V11  0x0201   /* ATMEGA644 Board V1.0, Antenna switch V 1.0 */
#define MT_TUNER_V10  0x0202   /* ATMEGA644 Board V1.1, Antenna tuner V 1.0 */
#define MT_TUNER_V12  0x0203   /* ATMEGA644 Board V1.1, Antenna tuner V 1.2 */
#define MT_FRQCNT_V11 0x0310   /* Frequency Counter V1.1 with ATMEGA644 Board V1.1 */
#define MT_FRQCNT_V20 0x0410   /* Frequency Counter V2.0 with integrated ATMEGA644 */

#define BOOTLOADER_TYPE 0x8000 /* or'd to previous types to mark bootloader */

/* Default Modbus device ID */
#define DDID_ANTSW_V10 17
#define DDID_ANTSW_V11 17
#define DDID_TUNER_V10 20
#define DDID_FRQCNT_V11 30
#define DDID_FRQCNT_V20 30

#define MT_COUNT 7              /* Number of supported modules, keep in sync with number of types above */

/* Define the available boards and their characteristics:
XTAL_FRQHZ:     nominal crystal frequency in Hz (uint32_t)
F_RT:           measured crystal frequency in Hz at room temperature (uint32_t)
tcxo_prec:      assumed precison of TCXO in ppb (uint32_t)
tcxo_a0:        parameter a0 of polynomial regression curve (float)
tcxo_a1:        parameter a1 of polynomial regression curve (float)
tcxo_a2:        parameter a2 of polynomial regression curve (float)
tcxo_a3:        parameter a3 of polynomial regression curve (float)
tcxo_a4:        parameter a4 of polynomial regression curve (float)
tcxo_a5:        parameter a5 of polynomial regression curve (float)

F_T = measured/real frequency at temperature T
DELTA_FT = XTAL_FRQHZ - F_T
DELTA_FT = a0 + a1*T + a2*T^2 + a3*T^3 + a4*T^4 + a5*T^5

*/

/* ATTINY1634 V1.0 */
#if (BOARD==1)
/* assembled device: TINY1634-U
   assembled voltage regulator: 5.0V */
#define HW_VERSION 1000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 8000000
#define F_RT 7999980
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

#if (BOARD==2)
/* assembled device: TINY1634R-SU
   assembled voltage regulator: 5.0V */
#define HW_VERSION 1000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 11059200
#define F_RT 11059180
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

#if (BOARD==3)
/* assembled device: TINY1634-U
   assembled voltage regulator: 5.0V */
#define HW_VERSION 1000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 12000000
#define F_RT 11999998
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

/* ATMEGA644 V1.0 */
#if (BOARD==11)
/* assembled device: ATMEGA644PA AU
   assembled voltage regulator: 5.0V
   temperature sensor: TMP275
   voltage reference: REF5025 */

/* NOTE: !!! DON'T USE THIS BOARD FOR SERIOUS APPLICATIONS !!!
   =====
   This board was used for debugging. Its FLASH memory and its EEPROM have been programmed many times.
*/
#define HW_VERSION 1000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 11059200
#define F_RT 11059213
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

#if (BOARD==12)
/* Antennenumschalter */
/* assembled device: ATMEGA644PA AU
   assembled voltage regulator: 5.0V
   temperature sensor: TMP275
   voltage reference: REF5025 */
#define HW_VERSION 1000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 14745600
#define F_RT 14745011
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

#if (BOARD==13)
/* Antennentuner, alt */
/* assembled device: ATMEGA644PA AU
   assembled voltage regulator: 5.0V
   temperature sensor: TMP275
   voltage reference: REF5025 */
#define HW_VERSION 1000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 18432000
#define F_RT 18431541
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

/* ATMEGA644 V1.1 */
#if (BOARD==21)
/* Antennentuner, neu, V1.2 */
/* assembled device: ATMEGA644PA AU
   assembled voltage regulator: 5.0V
   temperature sensor: TMP275
   voltage reference: REF5025 */
#define HW_VERSION 1100 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 18432000
#define F_RT 18431604
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

#if (BOARD==22)
/* assembled device: ATMEGA644V 10AU
   tool: select ATMEGA644 or ATMEGA644A
   assembled voltage regulator: 3.3V
   temperature sensor: TMP275
   voltage reference: none */
#define HW_VERSION 1100 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 8000000
#define F_RT 7999767
#define tcxo_prec 0
#define tcxo_a0 0.0
#define tcxo_a1 0.0
#define tcxo_a2 0.0
#define tcxo_a3 0.0
#define tcxo_a4 0.0
#define tcxo_a5 0.0
#endif

#if (BOARD==23)
/* assembled device: ATMEGA644PA AU
   assembled voltage regulator: 5.0V
   temperature sensor: TMP275
   voltage reference: REF5025 */
#define HW_VERSION 1100 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 18432000
#define F_RT 18431537
#define tcxo_prec 2
#define tcxo_a0 249.757365863836
#define tcxo_a1 3.94594089675493
#define tcxo_a2 6.37471219966487e-2
#define tcxo_a3 6.41133886903767e-5
#define tcxo_a4 -1.17467690924319e-5
#define tcxo_a5 0.0
#endif

/* Special Boards */
/* Frequency Counter V2.0 */
#if (BOARD==1001)
#define HW_VERSION 2000 /* Hardware version (16 bit). Displayed as nn.mmm */
#define XTAL_FRQHZ 100000000
#define CPU_INCLK_KHZ (XTAL_FRQHZ/10000) /* CPU clock input is XO/10 ! */
#define F_RT 99996621
#define tcxo_prec 2
#define tcxo_a0 2.924051677e3
#define tcxo_a1 -5.794266053
#define tcxo_a2 9.180343812e-1
#define tcxo_a3 -1.130409447e-2
#define tcxo_a4 5.202601599e-6
#define tcxo_a5 -3.992771202e-8
#endif

#define XTAL_FRQKHZ ((XTAL_FRQHZ+500)/1000)

#ifndef CPU_INCLK_KHZ
#define CPU_INCLK_KHZ ((XTAL_FRQHZ+500)/1000)
#endif

#if (XTAL_FRQHZ == F_RT)
#define RTC_CORRECTION 0 /* no clock deviation, no correction */
#else
#define RTC_CORRECTION ((50+(XTAL_FRQHZ/(XTAL_FRQHZ-F_RT)))/100)
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifdef WIN32

typedef struct {
        uint16_t moduletype ;   /* type of module according to definitions above */
        PTCHAR name ;           /* pointer to description */
        } MODULES ;

#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Inline functions                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifdef WIN32

#ifdef DECLARE_VARS
MODULES module_types[] = {{MT_ANTSW_V10,  _T("Antenna Switch V1.0 with ATTINY1634-RS485 Board V1.0")},
                          {MT_ANTSW_V11,  _T("Antenna Switch V1.0 with ATMEGA644-RS485 Board V1.0")},
                          {MT_TUNER_V10,  _T("Antenna Tuner V1.0 with ATMEGA644-RS485 Board V1.1")},
                          {MT_TUNER_V12,  _T("Antenna Tuner V1.2 with ATMEGA644-RS485 Board V1.1")},
                          {MT_FRQCNT_V11, _T("Frequency Counter V1.1 with ATMEGA644-RS485 Board V1.1")},
                          {MT_FRQCNT_V20, _T("Frequency Counter V2.0 with ATMEGA644 and USB Adapter")},
                          {MT_UNKNOWN, _T("unknown")},
                         } ;
#else
extern MODULES module_types[MT_COUNT] ;
#endif

#endif


#endif
