﻿/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|             File Name: UART_ATMEGA644.h                                                                               |
|                Author: Michael Krämer                                                                                 |
|                 Email: Michael@dk8pp.de                                                                               |
|                  Date: 23.01.2021                                                                                     |
|                  Tool: AVR Studio 7.0                                                                                 |
|                Device: ATMega644                                                                                      |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: include file for UART access                                                                                |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+

Preprocessor-Macros. Define the following preprocessor symbols to achieve the respective behavior:
====================

FULLDUPLEX:     Full duplex UART operation. RS485 driver is not switched on or off.
                otherwise: Half duplex UART operation. RS485 driver is switched on for transmit and off for receive.

*/
#ifndef UART_ACCESS

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define UART_ACCESS 1

#define RXBUFSIZE 255     /* size of RX buffer */
#define TXBUFSIZE 255     /* size of TX buffer */

#define BAUDRATE10TH 960 /* default baudrate is 9600 Bd */
#define EVEN_PARITY      /* use even parity */

#ifdef EVEN_PARITY
#define MYUPM01 (1<<UPM01)
#else
#define MYUPM01 (0<<UPM01)
#endif

/* Define UART flags to make access more handy */
#define RXCIE (1<<RXCIE0)
#define TXCIE (1<<TXCIE0)
#define UDRIE (1<<UDRIE0)
#define RXEN  (1<<RXEN0)
#define TXEN  (1<<TXEN0)
#define UCSZ2 (1<<UCSZ02)
#define TXB8  (1<<TXB80)

/* Predefined BRG and Timer1 settings (calculated with spreadsheet)
   UBBRVAL_n=0 means baudrate is unsupported (error > 0.5%)
   T15_n and T35_n refer to the recommended timeouts of the Modbus spec.
   The timeouts are in increments of Timer1, whose prescaler ist set to 64.
   Timer1 is also used to generate the 10 ms interval, therefore the OCR1A
   reload value is also defined here.
*/
#if (IO_CLK==8000)
/* 8,000 MHz */
#define UBRRVAL_110     4544
#define UBRRVAL_300     1666
#define UBRRVAL_600     832
#define UBRRVAL_1200    416
#define UBRRVAL_2400    207
#define UBRRVAL_4800    103
#define UBRRVAL_9600    51
#define UBRRVAL_14400   34
#define UBRRVAL_19200   25
#define UBRRVAL_28800   0
#define UBRRVAL_38400   12
#define UBRRVAL_57600   0
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         30682
#define T25_300         11250
#define T25_600         5625
#define T25_1200        2813
#define T25_2400        1406
#define T25_4800        703
#define T25_9600        352
#define T25_14400       234
#define T25_19200       176
#define T25_28800       141
#define T25_38400       130
#define T25_57600       118
#define T25_76800       112
#define T25_115200      106
#define T25_128000      104
#define T25_256000      99
#define T25_921600      95
#define T35_110         43182
#define T35_300         15833
#define T35_600         7917
#define T35_1200        3958
#define T35_2400        1979
#define T35_4800        990
#define T35_9600        495
#define T35_14400       330
#define T35_19200       247
#define T35_28800       219
#define T35_38400       219
#define T35_57600       219
#define T35_76800       219
#define T35_115200      219
#define T35_128000      219
#define T35_256000      219
#define T35_921600      219
#define PRSC_TCCR1B     3 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       1250
#endif

#if (IO_CLK==8333)
/* 8,333 MHz */
/* Imprecise 10ms interrupt! Deviation -32ppm=-2,768 sec/day */
#define UBRRVAL_110     0
#define UBRRVAL_300     0
#define UBRRVAL_600     0
#define UBRRVAL_1200    433
#define UBRRVAL_2400    216
#define UBRRVAL_4800    108
#define UBRRVAL_9600    53
#define UBRRVAL_14400   35
#define UBRRVAL_19200   26
#define UBRRVAL_28800   17
#define UBRRVAL_38400   0
#define UBRRVAL_57600   8
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         0
#define T25_300         0
#define T25_600         0
#define T25_1200        23437
#define T25_2400        11719
#define T25_4800        5859
#define T25_9600        2930
#define T25_14400       1953
#define T25_19200       1465
#define T25_28800       1179
#define T25_38400       1080
#define T25_57600       980
#define T25_76800       930
#define T25_115200      881
#define T25_128000      871
#define T25_256000      826
#define T25_921600      794
#define T35_110         0
#define T35_300         0
#define T35_600         0
#define T35_1200        32986
#define T35_2400        16493
#define T35_4800        8247
#define T35_9600        4123
#define T35_14400       2749
#define T35_19200       2062
#define T35_28800       1823
#define T35_38400       1823
#define T35_57600       1823
#define T35_76800       1823
#define T35_115200      1823
#define T35_128000      1823
#define T35_256000      1823
#define T35_921600      1823
#define PRSC_TCCR1B     2 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       10417
#endif

#if (IO_CLK==10000)
/* 10,000 MHz */
#define UBRRVAL_110     0
#define UBRRVAL_300     0
#define UBRRVAL_600     0
#define UBRRVAL_1200    520
#define UBRRVAL_2400    259
#define UBRRVAL_4800    129
#define UBRRVAL_9600    64
#define UBRRVAL_14400   42
#define UBRRVAL_19200   32
#define UBRRVAL_28800   21
#define UBRRVAL_38400   15
#define UBRRVAL_57600   10
#define UBRRVAL_76800   7
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         0
#define T25_300         0
#define T25_600         56250
#define T25_1200        28125
#define T25_2400        14063
#define T25_4800        7031
#define T25_9600        3516
#define T25_14400       2344
#define T25_19200       1758
#define T25_28800       1415
#define T25_38400       1296
#define T25_57600       1176
#define T25_76800       1117
#define T25_115200      1057
#define T25_128000      1045
#define T25_256000      991
#define T25_921600      952
#define T35_110         0
#define T35_300         0
#define T35_600         0
#define T35_1200        39583
#define T35_2400        19792
#define T35_4800        9896
#define T35_9600        4948
#define T35_14400       3299
#define T35_19200       2474
#define T35_28800       2188
#define T35_38400       2188
#define T35_57600       2188
#define T35_76800       2188
#define T35_115200      2188
#define T35_128000      2188
#define T35_256000      2188
#define T35_921600      2188
#define PRSC_TCCR1B     2 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       12500
#endif

#if (IO_CLK==11059)
/* 11,0592 MHz */
#define UBRRVAL_110     6283
#define UBRRVAL_300     2303
#define UBRRVAL_600     1151
#define UBRRVAL_1200    575
#define UBRRVAL_2400    287
#define UBRRVAL_4800    142
#define UBRRVAL_9600    71
#define UBRRVAL_14400   47
#define UBRRVAL_19200   35
#define UBRRVAL_28800   23
#define UBRRVAL_38400   17
#define UBRRVAL_57600   11
#define UBRRVAL_76800   8
#define UBRRVAL_115200  5
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         42415
#define T25_300         15552
#define T25_600         7776
#define T25_1200        3888
#define T25_2400        1944
#define T25_4800        972
#define T25_9600        486
#define T25_14400       324
#define T25_19200       243
#define T25_28800       196
#define T25_38400       179
#define T25_57600       163
#define T25_76800       154
#define T25_115200      146
#define T25_128000      144
#define T25_256000      137
#define T25_921600      132
#define T35_110         59695
#define T35_300         21888
#define T35_600         10944
#define T35_1200        5472
#define T35_2400        2736
#define T35_4800        1368
#define T35_9600        684
#define T35_14400       456
#define T35_19200       342
#define T35_28800       302
#define T35_38400       302
#define T35_57600       302
#define T35_76800       302
#define T35_115200      302
#define T35_128000      302
#define T35_256000      302
#define T35_921600      302
#define PRSC_TCCR1B     3 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       1728
#endif

#if (IO_CLK==12000)
/* 12,000 MHz */
#define UBRRVAL_110     6817
#define UBRRVAL_300     2499
#define UBRRVAL_600     1249
#define UBRRVAL_1200    624
#define UBRRVAL_2400    312
#define UBRRVAL_4800    155
#define UBRRVAL_9600    77
#define UBRRVAL_14400   51
#define UBRRVAL_19200   38
#define UBRRVAL_28800   25
#define UBRRVAL_38400   0
#define UBRRVAL_57600   12
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         46023
#define T25_300         16875
#define T25_600         8438
#define T25_1200        4219
#define T25_2400        2109
#define T25_4800        1055
#define T25_9600        527
#define T25_14400       352
#define T25_19200       264
#define T25_28800       212
#define T25_38400       194
#define T25_57600       176
#define T25_76800       167
#define T25_115200      159
#define T25_128000      157
#define T25_256000      149
#define T25_921600      143
#define T35_110         64773
#define T35_300         23750
#define T35_600         11875
#define T35_1200        5938
#define T35_2400        2969
#define T35_4800        1484
#define T35_9600        742
#define T35_14400       495
#define T35_19200       371
#define T35_28800       328
#define T35_38400       328
#define T35_57600       328
#define T35_76800       328
#define T35_115200      328
#define T35_128000      328
#define T35_256000      328
#define T35_921600      328
#define PRSC_TCCR1B     3 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       1875
#endif

#if (IO_CLK==12500)
/* 12,500 MHz */
#define UBRRVAL_110     0
#define UBRRVAL_300     0
#define UBRRVAL_600     0
#define UBRRVAL_1200    650
#define UBRRVAL_2400    325
#define UBRRVAL_4800    162
#define UBRRVAL_9600    80
#define UBRRVAL_14400   53
#define UBRRVAL_19200   40
#define UBRRVAL_28800   26
#define UBRRVAL_38400   0
#define UBRRVAL_57600   0
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         0
#define T25_300         0
#define T25_600         0
#define T25_1200        35156
#define T25_2400        17578
#define T25_4800        8789
#define T25_9600        4395
#define T25_14400       2930
#define T25_19200       2197
#define T25_28800       1769
#define T25_38400       1619
#define T25_57600       1470
#define T25_76800       1396
#define T25_115200      1321
#define T25_128000      1306
#define T25_256000      1239
#define T25_921600      1191
#define T35_110         0
#define T35_300         0
#define T35_600         0
#define T35_1200        49479
#define T35_2400        24740
#define T35_4800        12370
#define T35_9600        6185
#define T35_14400       4123
#define T35_19200       3092
#define T35_28800       2734
#define T35_38400       2734
#define T35_57600       2734
#define T35_76800       2734
#define T35_115200      2734
#define T35_128000      2734
#define T35_256000      2734
#define T35_921600      2734
#define PRSC_TCCR1B     2 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       15625
#endif

#if (IO_CLK==14746)
/* 14,7456 MHz */
#define UBRRVAL_110     0
#define UBRRVAL_300     3071
#define UBRRVAL_600     1535
#define UBRRVAL_1200    767
#define UBRRVAL_2400    383
#define UBRRVAL_4800    191
#define UBRRVAL_9600    95
#define UBRRVAL_14400   63
#define UBRRVAL_19200   47
#define UBRRVAL_28800   31
#define UBRRVAL_38400   23
#define UBRRVAL_57600   15
#define UBRRVAL_76800   11
#define UBRRVAL_115200  7
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         56553
#define T25_300         20736
#define T25_600         10368
#define T25_1200        5184
#define T25_2400        2592
#define T25_4800        1296
#define T25_9600        648
#define T25_14400       432
#define T25_19200       324
#define T25_28800       261
#define T25_38400       239
#define T25_57600       217
#define T25_76800       206
#define T25_115200      195
#define T25_128000      193
#define T25_256000      183
#define T25_921600      176
#define T35_110         65535 /* overflow! */
#define T35_300         29184
#define T35_600         14529
#define T35_1200        7296
#define T35_2400        3648
#define T35_4800        1824
#define T35_9600        912
#define T35_14400       608
#define T35_19200       456
#define T35_28800       403
#define T35_38400       403
#define T35_57600       403
#define T35_76800       403
#define T35_115200      403
#define T35_128000      403
#define T35_256000      403
#define T35_921600      403
#define PRSC_TCCR1B     3 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       2304
#endif

#if (IO_CLK==18432)
/* 18,432 MHz */
#define UBRRVAL_110     0
#define UBRRVAL_300     3839
#define UBRRVAL_600     1919
#define UBRRVAL_1200    959
#define UBRRVAL_2400    479
#define UBRRVAL_4800    239
#define UBRRVAL_9600    119
#define UBRRVAL_14400   79
#define UBRRVAL_19200   59
#define UBRRVAL_28800   39
#define UBRRVAL_38400   29
#define UBRRVAL_57600   19
#define UBRRVAL_76800   14
#define UBRRVAL_115200  9
#define UBRRVAL_128000  8
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         65535 /* overflow! */
#define T25_300         25920
#define T25_600         12960
#define T25_1200        6480
#define T25_2400        3240
#define T25_4800        1620
#define T25_9600        810
#define T25_14400       540
#define T25_19200       405
#define T25_28800       326
#define T25_38400       299
#define T25_57600       271
#define T25_76800       257
#define T25_115200      244
#define T25_128000      241
#define T25_256000      228
#define T25_921600      219
#define T35_110         65535 /* overflow! */
#define T35_300         36480
#define T35_600         18240
#define T35_1200        9120
#define T35_2400        4560
#define T35_4800        2280
#define T35_9600        1140
#define T35_14400       760
#define T35_19200       570
#define T35_28800       504
#define T35_38400       504
#define T35_57600       504
#define T35_76800       504
#define T35_115200      504
#define T35_128000      504
#define T35_256000      504
#define T35_921600      504
#define PRSC_TCCR1B     3 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       2880
#endif

#if (IO_CLK==20000)
/* 20,000 MHz */
#define UBRRVAL_110     0
#define UBRRVAL_300     4166
#define UBRRVAL_600     2082
#define UBRRVAL_1200    1041
#define UBRRVAL_2400    520
#define UBRRVAL_4800    259
#define UBRRVAL_9600    129
#define UBRRVAL_14400   86
#define UBRRVAL_19200   64
#define UBRRVAL_28800   42
#define UBRRVAL_38400   0
#define UBRRVAL_57600   0
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         65535 /* overflow! */
#define T25_300         28125
#define T25_600         14063
#define T25_1200        7031
#define T25_2400        3516
#define T25_4800        1758
#define T25_9600        879
#define T25_14400       586
#define T25_19200       439
#define T25_28800       354
#define T25_38400       324
#define T25_57600       294
#define T25_76800       279
#define T25_115200      264
#define T25_128000      261
#define T25_256000      248
#define T25_921600      238
#define T35_110         65535 /* overflow! */
#define T35_300         39583
#define T35_600         19792
#define T35_1200        9896
#define T35_2400        4948
#define T35_4800        2474
#define T35_9600        1237
#define T35_14400       825
#define T35_19200       618
#define T35_28800       547
#define T35_38400       547
#define T35_57600       547
#define T35_76800       547
#define T35_115200      547
#define T35_128000      547
#define T35_256000      547
#define T35_921600      547
#define PRSC_TCCR1B     3 /* Select prescaler: 1: clkIO/1; 2: clkIO/8; 3: clkIO/64; 4: clkIO/256; 5: clkIO/1024 */
#define RLD_OCR1A       3125
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
typedef struct rx_buffer {
        volatile uint8_t next ;         /* next in buffer */
        volatile uint8_t size ;         /* size of buffer */
        volatile uint8_t status ;       /* indicate reception status */
        volatile uint8_t bytecounter ;  /* counts the number of received bytes */
        volatile uint8_t buffer[RXBUFSIZE] ;   /* RX buffer */
        } RXBUFFER ;

typedef struct tx_buffer {
        volatile uint8_t next ;         /* next in buffer */
        volatile uint8_t count ;        /* number of characters to transmit */
        volatile bool completed ;       /* indicate transmission status */
        volatile uint8_t bytecounter ;  /* counts the number of transmitted bytes */
        volatile uint8_t buffer[TXBUFSIZE] ;   /* TX buffer */
        } TXBUFFER ;

typedef enum { /* Define receive states */
        RXSTATE_IDLE=0,                 /* idle, no specific state */
        RXSTATE_RECEIVING,              /* reception was started */
        RXSTATE_TIMEOUT,                /* character timeout, may become a frame timeout */
        RXSTATE_ERROR,                  /* receive error */
        RXSTATE_COMPLETED,              /* frame timeout, reception completed */
        RXSTATE_BUFFEROVERFLOW          /* receive buffer overflow */
        } RXSTATE_TYPE ;

typedef struct ct_struct {              /* communication timings */
        uint16_t ubrr ;                 /* UBRR0 register setting */
        uint16_t t25 ;                  /* t25 timeout, see ModBus spec */
        uint16_t t35 ;                  /* t35 timeout, see ModBus spec */
        uint16_t br10 ;                 /* baudrate / 10 */
        } COMM_TIMES ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_EXTERN_ RXBUFFER *prxb ; /* points to active receive buffer */
_EXTERN_ TXBUFFER *ptxb ; /* points to active transmit buffer */
_EXTERN_ RXBUFFER rxbuf ; /* receive buffer */
_EXTERN_ TXBUFFER txbuf ; /* transmit buffer */
_EXTERN_ uint16_t ubrrval ; /* baudrate */
_EXTERN_ uint16_t t25 ; /* communication timeout */
_EXTERN_ uint16_t t35 ; /* communication timeout */

/* NOINIT: the following variables survive the startup code */
/* Module status and error counters. May be read and written via ModBus functions */
_EXTERN_ volatile uint16_t com_error_count_h __attribute__ ((section (".noinit"))) ; /* Number of communication errors since last power-on reset (higher) */
_EXTERN_ volatile uint16_t com_error_count_l __attribute__ ((section (".noinit"))) ; /* Number of communication errors since last power-on reset (lower) */
_EXTERN_ volatile uint16_t com_rx_count_l __attribute__ ((section (".noinit"))) ; /* Number of received bytes since last power-on reset (lower) */
_EXTERN_ volatile uint16_t com_rx_count_h __attribute__ ((section (".noinit"))) ; /* Number of received bytes since last power-on reset (higher) */
_EXTERN_ volatile uint16_t com_tx_count_l __attribute__ ((section (".noinit"))) ; /* Number of transmitted bytes since last power-on reset (lower) */
_EXTERN_ volatile uint16_t com_tx_count_h __attribute__ ((section (".noinit"))) ; /* Number of transmitted bytes since last power-on reset (higher) */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       PROGMEM constants                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define CT_SIZE 17 /* size of COMM_TIMES table below. Keep in sync! */
_EXTERN_ const PROGMEM COMM_TIMES comm_times[CT_SIZE]
#ifdef DECLARE_VARS
      = { {UBRRVAL_110,    T25_110,    T35_110,       11},
          {UBRRVAL_300,    T25_300,    T35_300,       30},
          {UBRRVAL_600,    T25_600,    T35_600,       60},
          {UBRRVAL_1200,   T25_1200,   T35_1200,     120},
          {UBRRVAL_2400,   T25_2400,   T35_2400,     240},
          {UBRRVAL_4800,   T25_4800,   T35_4800,     480},
          {UBRRVAL_9600,   T25_9600,   T35_9600,     960},
          {UBRRVAL_14400,  T25_14400,  T35_14400,   1440},
          {UBRRVAL_19200,  T25_19200,  T35_19200,   1920},
          {UBRRVAL_28800,  T25_28800,  T35_28800,   2880},
          {UBRRVAL_38400,  T25_38400,  T35_38400,   3840},
          {UBRRVAL_57600,  T25_57600,  T35_57600,   5760},
          {UBRRVAL_76800,  T25_76800,  T35_76800,   7680},
          {UBRRVAL_115200, T25_115200, T35_115200, 11520},
          {UBRRVAL_128000, T25_128000, T35_128000, 12800},
          {UBRRVAL_256000, T25_256000, T35_256000, 25600},
          {UBRRVAL_921600, T25_921600, T35_921600,     0} }
#endif
        ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void init_UART(void) ;
void receive(RXBUFFER *rxb) ;
void transmit(TXBUFFER *txb) ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Inline functions                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifdef FULLDUPLEX
__inline static void RS485_TX_enable(void)  {} /* do nothing */
__inline static void RS485_TX_disable(void) {} /* do nothing */
#else
__inline static void RS485_TX_enable(void)  {PORTB |=  (1<<PB0) ;} /* PB0 high to enable RS485 transmitter */
__inline static void RS485_TX_disable(void) {PORTB &= ~(1<<PB0) ;} /* PB0 low to disable RS485 transmitter */
#endif
__inline static void UART_enable(uint8_t data) {UCSR0B |=  data ;} /* enable UART feature */
__inline static void UART_disable(uint8_t data){UCSR0B &= ~data ;} /* disable UART feature */

#endif
