﻿/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|             File Name: UART_ATTINY1634.h                                                                              |
|                Author: Michael Krämer                                                                                 |
|                 Email: Michael@dk8pp.de                                                                               |
|                  Date: 23.01.2021                                                                                     |
|                  Tool: AVR Studio 7.0                                                                                 |
|                Device: ATTiny1634                                                                                     |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: include file for UART access                                                                                |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef UART_ACCESS

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define UART_ACCESS 1

#define RXBUFSIZE 255     /* size of RX buffer */
#define TXBUFSIZE 255     /* size of TX buffer */

#define BAUDRATE BR_9600 /* default baudrate, index into "comm_times" table */
#define EVEN_PARITY      /* use even parity */

#ifdef EVEN_PARITY
#define MYUPM01 (1<<UPM01)
#else
#define MYUPM01 (0<<UPM01)
#endif

/* Define UART flags to make access more handy */
#define RXCIE (1<<RXCIE0)
#define TXCIE (1<<TXCIE0)
#define UDRIE (1<<UDRIE0)
#define RXEN  (1<<RXEN0)
#define TXEN  (1<<TXEN0)
#define UCSZ2 (1<<UCSZ02)
#define TXB8  (1<<TXB80)

/* Predefined BRG and Timer1 settings (calculated with spreadsheet)
   UBBRVAL_n=0 means baudrate is unsupported (error > 0.5%)
   T15_n and T35_n refer to the recommended timeouts of the Modbus spec.
   The timeouts are in increments of Timer1, whose prescaler ist set to 64.
   Timer1 is also used to generate the 10 ms interval, therefore the OCR1A
   reload value is also defined here.
*/
#if (XTAL_FRQ==8000)
/* 8,000 MHz */
#define UBRRVAL_110     4544
#define UBRRVAL_300     1666
#define UBRRVAL_600     832
#define UBRRVAL_1200    416
#define UBRRVAL_2400    207
#define UBRRVAL_4800    103
#define UBRRVAL_9600    51
#define UBRRVAL_14400   0
#define UBRRVAL_19200   25
#define UBRRVAL_28800   0
#define UBRRVAL_38400   12
#define UBRRVAL_57600   0
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         30682
#define T25_300         11250
#define T25_600         5625
#define T25_1200        2813
#define T25_2400        1406
#define T25_4800        703
#define T25_9600        352
#define T25_14400       234
#define T25_19200       176
#define T25_28800       141
#define T25_38400       130
#define T25_57600       118
#define T25_76800       112
#define T25_115200      106
#define T25_128000      104
#define T25_256000      99
#define T25_921600      95
#define T35_110         43182
#define T35_300         15833
#define T35_600         7917
#define T35_1200        3958
#define T35_2400        1979
#define T35_4800        990
#define T35_9600        495
#define T35_14400       330
#define T35_19200       247
#define T35_28800       219
#define T35_38400       219
#define T35_57600       219
#define T35_76800       219
#define T35_115200      219
#define T35_128000      219
#define T35_256000      219
#define T35_921600      219
#define RLD_OCR1A       1250
#endif

#if (XTAL_FRQ==11059)
/* 11,0592 MHz */
#define UBRRVAL_110     6283
#define UBRRVAL_300     2303
#define UBRRVAL_600     1151
#define UBRRVAL_1200    575
#define UBRRVAL_2400    287
#define UBRRVAL_4800    142
#define UBRRVAL_9600    71
#define UBRRVAL_14400   47
#define UBRRVAL_19200   35
#define UBRRVAL_28800   23
#define UBRRVAL_38400   17
#define UBRRVAL_57600   11
#define UBRRVAL_76800   8
#define UBRRVAL_115200  5
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         42415
#define T25_300         15552
#define T25_600         7776
#define T25_1200        3888
#define T25_2400        1944
#define T25_4800        972
#define T25_9600        486
#define T25_14400       324
#define T25_19200       243
#define T25_28800       196
#define T25_38400       179
#define T25_57600       163
#define T25_76800       154
#define T25_115200      146
#define T25_128000      144
#define T25_256000      137
#define T25_921600      132
#define T35_110         59695
#define T35_300         21888
#define T35_600         10944
#define T35_1200        5472
#define T35_2400        2736
#define T35_4800        1368
#define T35_9600        684
#define T35_14400       456
#define T35_19200       342
#define T35_28800       302
#define T35_38400       302
#define T35_57600       302
#define T35_76800       302
#define T35_115200      302
#define T35_128000      302
#define T35_256000      302
#define T35_921600      302
#define RLD_OCR1A       1728
#endif

#if (XTAL_FRQ==12000)
/* 12,000 MHz */
#define UBRRVAL_110     6817
#define UBRRVAL_300     2499
#define UBRRVAL_600     1249
#define UBRRVAL_1200    624
#define UBRRVAL_2400    312
#define UBRRVAL_4800    155
#define UBRRVAL_9600    77
#define UBRRVAL_14400   51
#define UBRRVAL_19200   38
#define UBRRVAL_28800   25
#define UBRRVAL_38400   0
#define UBRRVAL_57600   12
#define UBRRVAL_76800   0
#define UBRRVAL_115200  0
#define UBRRVAL_128000  0
#define UBRRVAL_256000  0
#define UBRRVAL_921600  0
#define T25_110         46023
#define T25_300         16875
#define T25_600         8438
#define T25_1200        4219
#define T25_2400        2109
#define T25_4800        1055
#define T25_9600        527
#define T25_14400       352
#define T25_19200       264
#define T25_28800       212
#define T25_38400       194
#define T25_57600       176
#define T25_76800       167
#define T25_115200      159
#define T25_128000      157
#define T25_256000      149
#define T25_921600      143
#define T35_110         64773
#define T35_300         23750
#define T35_600         11875
#define T35_1200        5938
#define T35_2400        2969
#define T35_4800        1484
#define T35_9600        742
#define T35_14400       495
#define T35_19200       371
#define T35_28800       328
#define T35_38400       328
#define T35_57600       328
#define T35_76800       328
#define T35_115200      328
#define T35_128000      328
#define T35_256000      328
#define T35_921600      328
#define RLD_OCR1A       1875
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
typedef struct rx_buffer {
        volatile uint8_t next ;         /* next in buffer */
        volatile uint8_t size ;         /* size of buffer */
        volatile uint8_t status ;       /* indicate reception status */
        volatile uint8_t bytecounter ;  /* counts the number of received bytes */
        volatile uint8_t buffer[RXBUFSIZE] ;   /* RX buffer */
        } RXBUFFER ;

typedef struct tx_buffer {
        volatile uint8_t next ;         /* next in buffer */
        volatile uint8_t count ;        /* number of characters to transmit */
        volatile bool completed ;       /* indicate transmission status */
        volatile uint8_t bytecounter ;  /* counts the number of transmitted bytes */
        volatile uint8_t buffer[TXBUFSIZE] ;   /* TX buffer */
        } TXBUFFER ;

typedef enum { /* Define receive states */
        RXSTATE_IDLE=0,                 /* idle, no specific state */
        RXSTATE_RECEIVING,              /* reception was started */
        RXSTATE_TIMEOUT,                /* character timeout, may become a frame timeout */
        RXSTATE_ERROR,                  /* receive error */
        RXSTATE_COMPLETED,              /* frame timeout, reception completed */
        RXSTATE_BUFFEROVERFLOW          /* receive buffer overflow */
        } RXSTATE_TYPE ;

typedef struct ct_struct {              /* communication timings */
        uint16_t ubrr ;                 /* UBRR0 register setting */
        uint16_t t25 ;                  /* t25 timeout, see ModBus spec */
        uint16_t t35 ;                  /* t35 timeout, see ModBus spec */
        } COMM_TIMES ;

typedef enum { /* Define baudrates as offset in "comm_times" table */
        BR_110=0,       /*  110 Bd */
        BR_300,         /*  300 Bd */
        BR_600,         /*  600 Bd */
        BR_1200,        /*  1200 Bd */
        BR_2400,        /*  2400 Bd */
        BR_4800,        /*  4800 Bd */
        BR_9600,        /*  9600 Bd */
        BR_14400,       /*  14400 Bd */
        BR_19200,       /*  19200 Bd */
        BR_28800,       /*  28800 Bd */
        BR_38400,       /*  38400 Bd */
        BR_57600,       /*  57600 Bd */
        BR_76800,       /*  76800 Bd */
        BR_115200,      /*  115200 Bd */
        BR_128000,      /*  128000 Bd */
        BR_256000,      /*  256000 Bd */
        BR_921600       /*  921600 Bd */
        } BAUDRATE_TYPE ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_EXTERN_ RXBUFFER *prxb ; /* points to active receive buffer */
_EXTERN_ TXBUFFER *ptxb ; /* points to active transmit buffer */
_EXTERN_ RXBUFFER rxbuf ; /* receive buffer */
_EXTERN_ TXBUFFER txbuf ; /* transmit buffer */
_EXTERN_ uint16_t ubrrval ; /* baudrate */
_EXTERN_ uint16_t t25 ; /* communication timeout */
_EXTERN_ uint16_t t35 ; /* communication timeout */

/* NOINIT: the following variables survive the startup code */
/* Module status and error counters. May be read and written via ModBus functions */
_EXTERN_ volatile uint16_t com_error_count_h __attribute__ ((section (".noinit"))) ; /* Number of communication errors since last power-on reset (higher) */
_EXTERN_ volatile uint16_t com_error_count_l __attribute__ ((section (".noinit"))) ; /* Number of communication errors since last power-on reset (lower) */
_EXTERN_ volatile uint16_t com_rx_count_l __attribute__ ((section (".noinit"))) ; /* Number of received bytes since last power-on reset (lower) */
_EXTERN_ volatile uint16_t com_rx_count_h __attribute__ ((section (".noinit"))) ; /* Number of received bytes since last power-on reset (higher) */
_EXTERN_ volatile uint16_t com_tx_count_l __attribute__ ((section (".noinit"))) ; /* Number of transmitted bytes since last power-on reset (lower) */
_EXTERN_ volatile uint16_t com_tx_count_h __attribute__ ((section (".noinit"))) ; /* Number of transmitted bytes since last power-on reset (higher) */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       PROGMEM constants                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_EXTERN_ const PROGMEM COMM_TIMES comm_times[]
#ifdef DECLARE_VARS
      = { {UBRRVAL_110, T25_110, T35_110},
          {UBRRVAL_300, T25_300, T35_300},
          {UBRRVAL_600, T25_600, T35_600},
          {UBRRVAL_1200, T25_1200, T35_1200},
          {UBRRVAL_2400, T25_2400, T35_2400},
          {UBRRVAL_4800, T25_4800, T35_4800},
          {UBRRVAL_9600, T25_9600, T35_9600},
          {UBRRVAL_14400, T25_14400, T35_14400},
          {UBRRVAL_19200, T25_19200, T35_19200},
          {UBRRVAL_28800, T25_28800, T35_28800},
          {UBRRVAL_38400, T25_38400, T35_38400},
          {UBRRVAL_57600, T25_57600, T35_57600},
          {UBRRVAL_76800, T25_76800, T35_76800},
          {UBRRVAL_115200, T25_115200, T35_115200},
          {UBRRVAL_128000, T25_128000, T35_128000},
          {UBRRVAL_256000, T25_256000, T35_256000},
          {UBRRVAL_921600, T25_921600, T35_921600} }
#endif
        ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void init_UART(void) ;
void receive(RXBUFFER *rxb) ;
void transmit(TXBUFFER *txb) ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Inline functions                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
__inline static void RS485_TX_enable(void)  {PORTA |=  (1<<PA0) ;} /* PA0 high to enable RS485 transmitter */
__inline static void RS485_TX_disable(void) {PORTA &= ~(1<<PA0) ;} /* PA0 low to disable RS485 transmitter */
__inline static void UART_enable(uint8_t data) {UCSR0B |=  data ;} /* enable UART feature */
__inline static void UART_disable(uint8_t data){UCSR0B &= ~data ;} /* disable UART feature */

#endif
