﻿/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|             File Name: Antennentuner.h                                                                                |
|                Author: Michael Krämer                                                                                 |
|                 Email: Michael@dk8pp.de                                                                               |
|                  Date: 26.02.2022                                                                                     |
|                  Tool: AVR Studio 7.0 and Visual Studio                                                               |
|                Device: ATMEGA                                                                                         |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Private include file for Antennentuner (used for host and remote software)                                  |
|           Defines modbus addresses and data structures unique for this module                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef ANTENNENTUNER
#define ANTENNENTUNER 1

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes for host and remote software                                                                           |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include "ModbusCommon.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines for host and remote software                                                                            |
+-----------------------------------------------------------------------------------------------------------------------+
*/
/* Private MCB EEPROM registers - none - */
enum {MCB_COUNT_TUNER=MCBC_COUNT  /* Number of MCB registers */
      } ;

/* User function Holding Registers starting at MA_FUNC */
enum {FCRO_RELAYS_I=0,  /* Function register to set relays for inductors */
      FCRO_RELAYS_C,    /* Function register to set relays for capacitors */
      FCR_COUNT} ;      /* Number of frequency counter registers */

#ifdef BOOTLOADER
#define MB_NORC         /* No "read coils" supported (function code 0x01) */
//#define MB_NORR         /* No "read input/holding registers" supported (function codes 0x03/0x04) */
#define MB_NOWSC        /* No "write single coil" supported (function code 0x05) */
//#define MB_NOWSHR       /* No "Write single holding register" supported (function code 0x06) */
//#define MB_NOWMR        /* No "Write multiple holding registers" supported (function code 0x10) */
#define MB_NORFR        /* No "Read File Record" supported (function code 0x14) */

#define BAUDRATEINDEX 6  /* default baudrate is 9600 Bd */
#endif

#ifndef WIN32
/* for remote software only */
/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Early Defines (needed by the include files below)                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef MODULE_TYPE
//#define MODULE_TYPE MT_TUNER_V10 /* Tuner version 1.0 or V1.1 */
#define MODULE_TYPE MT_TUNER_V12 /* Tuner version 1.2 */
#endif

#ifndef BOARD
#if (MODULE_TYPE==MT_TUNER_V10)
#define BOARD 13
#else
#define BOARD 21
#endif
#endif

#ifndef _EXTERN_
   #ifdef DECLARE_VARS
     #define _EXTERN_
     #define _EQ_(val) = val
     #define _EQ4_(v1,v2,v3,v4) = {v1,v2,v3,v4}
     #define _EQ6_(v1,v2,v3,v4,v5,v6) = {v1,v2,v3,v4,v5,v6}
     #define _EQ8_(v1,v2,v3,v4,v5,v6,v7,v8) = {v1,v2,v3,v4,v5,v6,v7,v8}
   #else
     #define _EXTERN_ extern
     #define _EQ_(val)
     #define _EQ4_(v1,v2,v3,v4)
     #define _EQ6_(v1,v2,v3,v4,v5,v6)
     #define _EQ8_(v1,v2,v3,v4,v5,v6,v7,v8)
   #endif
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include "ModbusBoards.h"
#define CPU_CLK  CPU_INCLK_KHZ /* CPU clock speed in kHz */
#define IO_CLK   CPU_INCLK_KHZ /* I/O clock speed in kHz */
#include "UART_ATMEGA644.h"
#include "modbus.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Configuration                                                                                                   |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef MODBUS_DEVICEADDR
#define MODBUS_DEVICEADDR DDID_TUNER_V10 /* Default Modbus address of this device */
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef BOOTLOADER
#define USE_TEMPSENSOR
#define USE_ADC
#define ADC_READ_INTERVAL 500    /* ADC read interval in cs */
#define TS_READ_INTERVAL  200    /* Temperature sensor read interval in cs */
#endif

#define NS_PER_CPUCLK (1000000l / CPU_CLK) /* nanoseconds per CPU clock cycle */
#define SW_VERSION 1100 /* Software version (16 bit). Displayed as nn.mmm */

#define RELAY_SET_TIME 3       /* time to switch relay on or off in cs */
#define RELAY_RECOVERY_TIME 1  /* recovery time between switching relays in cs */

/*
Port Pin connections of
ATMEGA644PA-AU Board with RS485 Interface
=========================================

        |           |     | ATM644 | ATM644 | Driver | Driver |
        | dedicated | I/O | Device | Board  | Board  | Board  | Antenna tuner
Port    | Signal    | cfg |  Pin   |  Pin   |  Pin   | Signal | used for
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PA0     |     -     |  I  |    37  |    3   |        |        | Measure VIN via 4k7 - 1k voltage divider
PA1     |     -     |  O  |    36  |    4   |        |        | unused
PA2     |     -     |  O  |    35  |    5   |        |        | unused
PA3     |     -     |  O  |    34  |    6   |        |        | unused
PA4     |     -     |  O  |    33  |    7   |        |        | unused
PA5     |     -     |  O  |    32  |    8   |        |        | unused
PA6     |     -     |  O  |    31  |    9   |        |        | unused
PA7     |     -     |  O  |    30  |   10   |        |        | unused
PB0     |  DE       |  O  |    40  |    -   |        |        | RS485 driver enable
PB1     |     -     |  O  |    41  |   32   |   35   |  QH0   | SET1, Spulen
PB2     |     -     |  O  |    42  |   30   |   33   |  QH2   | RES1, Spulen
PB3     |  SCL      |  O  |    43  |   29   |        |        | Temperature sensor
PB4     |  SDA      |  O* |    44  |    -   |        |        | Temperature sensor
PB5     |  MOSI     |  I* |     1  |   27   |        |        | unused
PB6     |  MISO     |  O  |     2  |   28   |   32   |  QH3   | RES2, Kondensatoren
PB7     |  SCK      |  O  |     3  |   31   |   34   |  QH1   | SET2, Kondensatoren
PC0     |     -     |  O  |    19  |   19   |        |  QL0   | REL1
PC1     |     -     |  O  |    20  |   18   |        |  QL1   | REL2
PC2     |     -     |  O  |    21  |   17   |        |  QL2   | REL3
PC3     |     -     |  O  |    22  |   16   |        |  QL3   | REL4
PC4     |     -     |  O  |    23  |   15   |        |  QL4   | REL5
PC5     |     -     |  O  |    24  |   14   |        |  QL5   | REL6
PC6     |     -     |  O  |    25  |   13   |        |  QL6   | V1.2: REL7; V1.0: unused, not connected
PC7     |     -     |  O  |    26  |   11   |        |  QL7   | V1.2: REL8; V1.0: unused
PD0     |  RXD0     |  I  |     9  |    -   |        |        | Host I/F
PD1     |  TXD0     |  O  |    10  |    -   |        |        | Host I/F
PD2     |     -     |  O  |    11  |   25   |        |        | unused, not connected
PD3     |     -     |  O  |    12  |   24   |        |        | unused, not connected
PD4     |     -     |  O  |    13  |   23   |        |        | unused, not connected
PD5     |     -     |  O  |    14  |   22   |        |        | unused, not connected
PD6     |     -     |  O  |    15  |   21   |        |        | unused, not connected
PD7     |     -     |  O  |    16  |   20   |        |        | unused, not connected


Nominale Kapazitäten und Induktivitäten:

Relais auf Board V1.0:
======================
PC0=REL1, Relais K1 & K2,     5pF,  160nH
PC1=REL2, Relais K9 & K4,    10pF, 2560nH
PC2=REL3, Relais K5 & K6,    20pF,  640nH
PC3=REL4, Relais K3 & K8,    40pF, 5120nH
PC4=REL5, Relais K7 & K10,   80pF, 1280nH
PC5=REL6, Relais K11 & K12, 470pF,  320nH

Relais auf Board V1.2:
======================
PC0=REL1, Relais K1 & K13,  2,5pF,   80nH
PC1=REL2, Relais K2 & K14,    5pF,  320nH
PC2=REL3, Relais K3 & K15,   10pF,  160nH
PC3=REL4, Relais K4 & K16,   20pF,  640nH
PC4=REL5, Relais K5 & K11,   40pF, 1280nH
PC5=REL6, Relais K6 & K10,   80pF, 2560nH
PC6=REL7, Relais K7 & K9,   160pF, 5120nH
PC7=REL8, Relais K12 & K8,  330pF, 1024nH

I*: Input with internal pull-up
O*: Open-drain output

Ruhezustand: PC0..7 und PB1/2/6/7 auf low

Einschalten eines Relais: SET1 (Spulen) oder SET2 (Kondensatoren) auf high (=PB1/PB7 high), REL1..8 auf low (=PC0..7 high)
Ausschalten eines Relais: RES1 (Spulen) oder RES2 (Kondensatoren) auf high (=PB2/PB6 high), REL1..8 auf low (=PC0..7 high)

Ein 10ms Puls ist ausreichend zum Ein- oder Ausschalten

*/

/* Port settings according to previous table */
#define PORTA_DIR    0xfe /* PA7..PA1 are outputs */
#define PORTA_PULLUP 0x00 /* no pullups */
#define PORTB_DIR    0xdf /* PB7, PB6 and PB4..PB0 are outputs */
#define PORTB_PULLUP 0x30 /* PB5..PB4 have pullups */
#define PORTC_DIR    0xff /* PC7..PC0 are outputs */
#define PORTC_PULLUP 0x00 /* no pullups */
#define PORTD_DIR    0xfe /* PD7..PD1 are outputs, PD0 is input */
#define PORTD_PULLUP 0x00 /* no pullups */

#define VALID 0x3d      /* Valid marker for MCB (don't use 0x00 or 0xff, any other value should be fine) */

/* Pin & Port definitions for relays */
#define PORT_HIGHSIDE PORTB /* Port to drive high voltage */
#define PORT_LOWSIDE PORTC  /* Port to drive low voltage */

#if (MODULE_TYPE==MT_TUNER_V10)
#define PIN_LOWDRIVERS ((0<<PC7) | (0<<PC6) | (1<<PC5) | (1<<PC4) | (1<<PC3) | (1<<PC2) | (1<<PC1) | (1<<PC0)) /* bit map of all low drivers */
#endif
#if (MODULE_TYPE==MT_TUNER_V12)
#define PIN_LOWDRIVERS ((1<<PC7) | (1<<PC6) | (1<<PC5) | (1<<PC4) | (1<<PC3) | (1<<PC2) | (1<<PC1) | (1<<PC0)) /* bit map of all low drivers */
#endif

#define PIN_CRELAY_ON  (1<<PB7) /* Bit number of port B to switch high side relay driver for capacitors on (PB7) */
#define PIN_CRELAY_OFF (1<<PB6) /* Bit number of port B to switch high side relay driver for capacitors off (PB6) */
/* Note: the logic for the inductor relays is reversed, because an inductor is active, when the relay is off */
#define PIN_IRELAY_ON  (1<<PB2) /* Bit number of port B to switch high side relay driver for inductors on (PB2) */
#define PIN_IRELAY_OFF (1<<PB1) /* Bit number of port B to switch high side relay driver for inductors off (PB1) */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
typedef struct mcb_struct { /* main control block */
        uint8_t valid ;                 /* This MCB is valid if valid == VALID, any other value means block is invalid.
                                           This must be the FIRST element in the structure! */
        uint8_t numreg ;                /* Number of MCB registers */
/* the following data is common to all applications */
        uint8_t module_address ;        /* modbus address of this module */
        uint16_t br10th ;               /* 1/10 of the baudrate */
        int16_t rtc_correction ;        /* correction for RTC */
        uint16_t pgm_cntrl ;            /* counter for number of EEPROM programming cycles, lower */
        uint16_t pgm_cntrh ;            /* counter for number of EEPROM programming cycles, higher */
/* the following data is unique to this module */
                                        /* none */
/* end of unique section */
        uint16_t crc ;                  /* CRC on all previous elements in MCB, algorithm as for Modbus.
                                           This must be the LAST element in the structure! */
        } MCB ;

typedef struct tim {
        volatile uint16_t timer ;       /* down counter */
        volatile bool active ;          /* status */
        } TIMER ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void wait_for_rx_user(void) ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifdef DECLARE_VARS
const PROGMEM MCB default_mcb = {/* Master Control Block */
        .valid = VALID,                         /* this block is valid */
        .numreg = MCB_COUNT_TUNER,              /* Number of MCB registers */
        .module_address = MODBUS_DEVICEADDR,    /* modbus address of this module */
        .br10th = BAUDRATE10TH,                 /* 1/10th of the desired baudrate */
        .rtc_correction = RTC_CORRECTION,       /* default RTC correction */
        .pgm_cntrl = 1,                         /* lower programming counter = 1 */
        .pgm_cntrh = 0,                         /* higher programming counter = 0 */
        .crc = 0                                /* unused, will be calculated at runtime */
        } ;
#else
_EXTERN_ const PROGMEM MCB default_mcb ; /* Master Control Block */
#endif

/* Timer */
_EXTERN_ TIMER relay_timer ;   /* timer for relay switching sequence */

#ifdef USE_TEMPSENSOR
_EXTERN_ TIMER ts_timer ;   /* timer for temperature sensor */
#endif

#ifdef USE_ADC
_EXTERN_ TIMER adc_timer ;  /* timer for ADC */
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       EEPROM data                                                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef BOOTLOADER
_EXTERN_ MCB EEMEM eemcb[64] ;
_EXTERN_ TCXO EEMEM tcxo_parms _EQ8_ (XTAL_FRQHZ,tcxo_prec,tcxo_a0,tcxo_a1,tcxo_a2,tcxo_a3,tcxo_a4,tcxo_a5) ;
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Inline functions                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
__inline static void user_int10ms(void) /* user specific 10ms interrupt handler */
{ /* executed in interrupt context */
   if (relay_timer.active)
   {
      if (relay_timer.timer != 0) relay_timer.timer-- ;
      if (relay_timer.timer == 0) relay_timer.active = false ;
   }
}

#endif /* ifndef WIN32 */

#endif
